require 'crowdfund/project'
require 'crowdfund/pledge_pool'
require 'csv'

module Crowdfund
	describe Project do

		before do
			$stdout = StringIO.new
			@current_funding = 100
			@initial_target = 1000
			@project = Project.new("abc", @current_funding, @initial_target)
		end

		it "has an initial target funding amount" do
			@project.goal.should == @initial_target
		end

		it "computes the total funding outstanding as the target funding amount minus the funding amount" do
			@project.funds_still_needed == @initial_target - @current_funding
		end

		it "increases funds by 25 when funds are added" do
			@project.add

			@project.funds.should == @current_funding + 25
	    end		

		it "decreases funds by 15 when funds are removed" do
			@project.subtract

			@project.funds.should == @current_funding - 15		
		end

		context "Project created with a default funding amount" do
			before do
				@project = Project.new("def")
			end

			it "has a default value of 0 for funding amount" do
				@project.funds.should == 0
			end
		end

		context "Project created with funding greater than the goal" do
			before do 
				$stdout = StringIO.new
				@current_funding = 1001
				@initial_target = 1000
				@project = Project.new("abc", @current_funding, @initial_target)	
			end

			it "has more funds than the goal" do 
				@project.funds.should > @project.goal
			end

			it "is fully funded" do
				@project.fully_funded?.should == true
			end
		end

		context "Project created with funding equal to the goal" do
			before do 
				$stdout = StringIO.new
				@current_funding = 1000
				@initial_target = 1000
				@project = Project.new("abc", @current_funding, @initial_target)	
			end

			it "funds have met the goal" do 
				@project.funds.should == @project.goal
			end

			it "is fully funded" do
				@project.fully_funded?.should == true
			end		
		end

		context "Project created with funding less than the goal" do
			before do 
				$stdout = StringIO.new
				@current_funding = 999
				@initial_target = 1000
				@project = Project.new("abc", @current_funding, @initial_target)	
			end

			it "has not met the goal" do 
				@project.funds.should < @project.goal
			end

			it "is not fully funded yet" do
				@project.fully_funded?.should == false
			end
		end	

		it "computes the sum of all pledges" do
			@project.pledge_amounts.should == 0

			@project.received_pledge(Pledge.new(:bronze,50))

			@project.pledge_amounts.should == 50

			@project.received_pledge(Pledge.new(:platinum,500))

			@project.pledge_amounts.should == 550

			@project.received_pledge(Pledge.new(:bronze,50))

			@project.pledge_amounts.should == 600
		end

		it "yields the total pledges for each pledge level" do
			@project.received_pledge(Pledge.new(:bronze, 50))
			@project.received_pledge(Pledge.new(:bronze, 50))
			@project.received_pledge(Pledge.new(:silver, 75))
			@project.received_pledge(Pledge.new(:silver, 75))
			@project.received_pledge(Pledge.new(:gold, 100))
			@project.received_pledge(Pledge.new(:gold, 100))
			@project.received_pledge(Pledge.new(:platinum, 500))
			@project.received_pledge(Pledge.new(:platinum, 500))

			yielded = []
			@project.each_pledge_level do |pledge|
				yielded << pledge
			end

			yielded.should == [
				Pledge.new(:bronze, 100),
				Pledge.new(:silver, 150),
				Pledge.new(:gold, 200),
				Pledge.new(:platinum, 1000)
			]
		end

		it "can be created from a CSV string" do
			project = Crowdfund::Project.new("csv", 1000, 100)

			project.name.should == "CSV"
			project.funds.should == 1000
			project.goal.should == 100
		end

	end	
end